{$DEFINE DSound}
{*$UNDEF DSound*}

{Version 1.0}

Unit Sound;
Interface
{$IFDEF DSound}
Uses DSound,MMSystem,Forms;
{$ELSE}
Uses MMSystem;
{$ENDIF}

Const
  SND_FLUSHDONE=1; // le flush est termin


Type
  TInfoBuff=Record // proprits du buffer
    Nfo:Pointer; // pointeur vers la zone mmoire utilisateur
    Bytes:LongWord; // offset global du son contenu dans le buffer
  End;
  PInfoBuff=^TInfoBuff;

  TDecHandler=Procedure(Pntr,Nfo:Pointer); // Dclaration decodeur
  TErrHandler=Procedure(Err1,Err2:LongWord;Txt:String); // Dclaration Erreur
  TMsgHandler=Procedure(Msg,Param1,Param2:LongWord); // Dclaration Message

  TSound=Class(TObject)

    Private
      LastReserve:Word;                               // reserves au moment du dernier flush
      InPre:Boolean;                                  // flag de prbuffering
      Errh:Boolean;                                   // flag d'erreur
      InWav:Boolean;                                  // flag de dcodage
      InfoSize:LongWord;                              // taille mmoire utilisateur
      LastBuffer:Word;                                // dernier buffer en cours trouv
      Stereo:Boolean;                                 // flag stro
      B16:Boolean;                                    // flag 16 bits
      Freq:Word;                                      // frquence
      CurrBytes:Longword;                             // octets courants
      Joue:Boolean;                                   // flag priph on/off
      TailleBuffer:LongWord;                          // taille buffer
      MaxBuffs:Word;                                  // nombre de buffers
      DevIndex:LongWord;                              // index du device
      DevHandle:LongWord;                             // handle du device
      DevType:Word;                                   // type de device
      ByteCount:LongWord;                             // octets dcods
      OutFName:String;                                // fichier de sortie sur disque
      OutFile:File;                                   // handle du fichier de sortie
      WaveHeader:Array Of WaveHdr;                    // buffers
      DecHandler:TDecHandler;                         // handler decodage
      ErrHandler:TErrHandler;                         // handler erreur
      MsgHandler:TMsgHandler;                         // handler message
      IsPause:Boolean;                                // flag pause
      Pre:Single;                                     // prebuffering
      InFlush:Boolean;                                // flag de flush
      FlushDoneAction:Byte;                           // action de fin de flush
      NextTailleBuffer:LongWord;                      // prochaine taille de buffer
      NextMaxBuffs:Word;                              // prochain nombre de buffers
      NextDevType:Word;                               // prochain type de priph
      NextDevIndex:LongWord;                          // prochain index de priph
      NextFreq:Word;                                  // prochaine frquence
      NextStereo:Boolean;                             // prochain flag stro
      NextB16:Boolean;                                // prochain flag 16 bits
      NextOutFName:String;                            // prochain fichier de sortie
      NextInfoSize:LongWord;                          // prochaine taille mmoire utilisateur
      NextPre:Single;                                 // prochain pr-buffering
      NextVol:LongWord;                               // prochain volume
{$IFDEF DSound}
      ppDS:IDirectSound;                              // objet directsound
      lpIDirectSoundBuffer:IDirectSoundBuffer;        // buffer directsound
      lpIDirectSoundBufferPrimary:IDirectSoundBuffer; // primary buffer directsound
      NextBufferToWrite:Word;                         // prochain buffer en criture
{$ENDIF}
      Function GetError(Param:Longword):String;       // texte erreur du mappeur
      Procedure SndStop;                              // stop
      Procedure Do_Wav;                               // perform
      Function Init_Snd:Boolean;                      // play
      Procedure SetSb(Cmd:Integer;Param:LongWord);    // modification de params
      Function GetSb(Cmd:Integer):LongWord;           // retourne une info
      Function GetSbP(Cmd:Integer):Pointer;           // retourne une info pointeur
      Function GetNfo(NBuff:LongWord):Pointer;        // espace utilisateur
      Procedure Error(N1,N2:Byte;Txt:String);         // gnre l'erreur
      Procedure SetVolume(Vol:LongWord);              // modifie le volume
      Function GetVolume:LongWord;                    // retourne le volume
    Public
      Property DeviceType:Word Read DevType Write NextDevType;
      Property DeviceIndex:LongWord Read DevIndex Write NextDevIndex;
      Property OutFrequency:Word Read Freq Write NextFreq;
      Property OutStereo:Boolean Read Stereo Write NextStereo;
      Property Out16Bits:Boolean Read B16 Write NextB16;
      Property OutFileName:String Read OutFName Write NextOutFName;
      Property InfoStructureSize:LongWord Read InfoSize Write NextInfoSize;
      Property BufferSize:LongWord Read TailleBuffer Write NextTailleBuffer;
      Property BufferCount:Word Read MaxBuffs Write NextMaxBuffs;
      Property PreBuffering:Single Read Pre Write NextPre;
      Property Volume:LongWord Read GetVolume Write SetVolume;

      Property DeviceHandle:LongWord Read DevHandle;
      Property Info[BufferIndex:LongWord]:Pointer Read GetNfo;
      Property PlayedBytes:LongWord Index 3 Read GetSb;
      Property CurrentBufferIndex:LongWord Index 5 Read GetSb;
      Property OffsetInCurrentBuffer:LongWord Index 6 Read GetSb;
      Property BuffersReady:LongWord Index 9 Read GetSb;
      Property CurrentBufferAddress:Pointer Index 10 Read GetSbP;
      Property Timer:LongWord Index 13 Read GetSb;
      Property Playing:Boolean Read Joue;
      Property Paused:Boolean Read IsPause;
      Property DecodedBytes:LongWord Read ByteCount;

      Property DecodeHandler:TDecHandler Read DecHandler Write DecHandler;
      Property ErrorHandler:TErrHandler Read ErrHandler Write ErrHandler;
      Property MessageHandler:TMsgHandler Read MsgHandler Write MsgHandler;

      Constructor Create;
      Function Play:Boolean;
      Procedure Stop;
      Procedure Pause;
      Procedure Resume;
      Procedure Perform;
      Procedure Update;
      Procedure FlushBuffers;
      Procedure FloodFill;
      Procedure Continue;
      Procedure FlushThenStop;
      Procedure FlushThenUpdate;
      Destructor Destroy;Override;

  End;

Implementation

Procedure TSound.SetVolume(Vol:LongWord);
Begin
  NextVol:=Vol; // on sauve la valeur si jamais on ne peut rien en faire maintenant
  If Not Joue Then Exit;
  Case DevType Of
    0:waveOutSetVolume(devhandle,vol+vol shl 16);
{$IFDEF DSound}
    2:lpidirectsoundbuffer.setvolume(trunc(10000/ln(65536)*ln(vol+1)-10000));
{$ENDIF}
  End;
End;

Function TSound.GetVolume:LongWord;
Var Rep:LongWord;
{$IFDEF DSound}
    iRep:Integer;
{$ENDIF}
Begin
  Result:=0;
  If Not Joue Then Begin
    Result:=NextVol; // la valeur retenue est retourne si on ne joue pas
    Exit;
  End;
  Case DevType Of
    0:Begin
      waveOutGetVolume(devhandle,@rep);
      Result:=rep and 65535;
    End;
    1:Begin
      Result:=65535; // pas de volume pour la sortie fichier
    End;
{$IFDEF DSound}
    2:Begin
      lpidirectsoundbuffer.getvolume(irep);
      Result:=trunc(exp((irep+10000)/(10000/ln(65536)))-1);
    End;
{$ENDIF}
  End;
End;

Function TSound.GetError(Param:LongWord):String;
Var Txt:Array[0..255] Of Char;
Begin
  waveOutGetErrorText(Param,@Txt[0],255);
  GetError:=Txt;
End;

Procedure TSound.FloodFill;
Var I:Integer;
Begin
  For I:=1 To MaxBuffs Do Perform;
End;

Procedure TSound.FlushThenStop;
Begin
  FlushDoneAction:=1;
  InFlush:=True;
  LastReserve:=GetSb(9); // rserves actuelles
  If InWav Then Inc(LastReserve); // si on tait dans le dcodage, a va en faire un de plus
End;

Procedure TSound.FlushThenUpdate;
Begin
  FlushDoneAction:=2;
  InFlush:=True;
  LastReserve:=GetSb(9); // rserves actuelles
  If InWav Then Inc(LastReserve); // si on tait dans le dcodage, a va en faire un en plus
End;


Procedure TSound.FlushBuffers;
Begin
  FlushDoneAction:=0;
  InFlush:=True;
  LastReserve:=GetSb(9); // rserves actuelles
  If InWav Then Inc(LastReserve); // si on tait dans le dcodage, a va en faire un en plus
End;

Procedure TSound.Continue;
Begin
  InFlush:=False;
End;

Function TSound.Play:Boolean;
Begin
  Result:=Init_Snd;
End;

Procedure TSound.Stop;
Begin
  SndStop;
End;

Procedure TSound.Pause;
Begin
  If Not IsPause Then SetSb(2,1);
End;

Procedure TSound.Resume;
Begin
  If IsPause Then SetSb(2,0);
End;

Procedure TSound.Perform;
Var Reserve:Word;
Begin
  If Not InFlush Then Begin // on ne perform plus dans le flush
    Do_Wav;
  End Else Begin
    Reserve:=GetSb(9);
    If (Reserve=0) Or (Reserve>LastReserve) Then Begin // si nos rserves sont finies
      Case FlushDoneAction Of                          // ou si le direct sound  boucl
        1:Begin
          Stop;
        End;
        2:Begin
          Update;
          InFlush:=False;
        End;
      End;
      MsgHandler(SND_FLUSHDONE,0,0);
    End;
  End;
End;

Procedure TSound.Update;
Begin
  If DevType<>1 Then Begin
    Stop;
    Play;
  End Else Begin
    Error(7,0,'This device type doesn''t support update');
  End;
End;

Destructor TSound.Destroy;
Begin
  If Joue Then SndStop;
  InHerited Destroy;
End;

{S'occupe de l'appel du handler des erreurs}
Procedure TSound.Error(N1,N2:Byte;Txt:String);
Begin
  If Not(ErrH) Then Begin
    ErrH:=True;
    ErrHandler(N1,N2,Txt);
    ErrH:=False;
  End;
End;

Function TSound.GetNfo(NBuff:LongWord):Pointer;
Begin
  GetNfo:=PInfoBuff(WaveHeader[NBuff-1].dwUser)^.NFo;
End;

Procedure TSound.SetSb(Cmd:Integer;Param:LongWord);
Begin
  Case Cmd Of
    (*pause on/off*)
    2:If Param=0 Then Begin // pause off
      Case DevType Of
        0:Begin
          IsPause:=False;
          waveOutRestart(DevHandle)
        End;
        1:Begin
          IsPause:=False;
        End;
{$IFDEF DSound}
        2:Begin
          IsPause:=False;
          lpIDirectSoundBuffer.Play(0,0,DSBPLAY_LOOPING);
        End;
{$ENDIF}
      End;
    End Else Begin // pause on
      Case DevType Of
        0:IsPause:=waveOutPause(DevHandle)=0;
        1:IsPause:=True;
{$IFDEF DSound}
        2:IsPause:=lpIDirectSoundBuffer.Stop=DS_OK;
{$ENDIF}
      End;
    End;
  End;
End;

Function TSound.GetSbP(Cmd:Integer):Pointer;
Var I:LongInt;
Begin
  GetSbP:=Nil;
  Case Cmd Of
    (*get buffofs*)
    10:Begin
      If Joue Then Begin
        I:=GetSb(5);
        GetSbP:=WaveHeader[I-1].lpData;
      End;
    End;
  End;
End;

Function TSound.GetSb(Cmd:Integer):LongWord;
Var Rep:LongWord;
Var I:LongInt;
Var Adr:LongWord;
    MMTime:TMMTime;
{$IFDEF DSound}
Var Curplay:Longword;
{$ENDIF}
Begin
  GetSb:=0;
  Case Cmd Of
    (*getcurrplayed*)
    3:Begin
      If Joue Then Begin
        Rep:=0;
        Case DevType Of
          0:Rep:=GetSb(4);
          1:Rep:=ByteCount;
{$IFDEF DSound}
          2:Rep:=ByteCount-GetSb(9)*TailleBuffer;
{$ENDIF}
        End;
        GetSb:=Rep;
      End;
    End;
    (*getcurrmcipos*)
    4:Begin
      If Joue Then Begin
        Rep:=0;
        Case DevType Of
          0:Begin
            MMTime.wType:=TIME_Bytes;
            waveOutGetPosition(DevHandle,@MMTime,23);
            Rep:=MMTime.cb;
          End;
          1:Begin
            Rep:=ByteCount;
          End;
{$IFDEF DSound}
          2:Begin
            lpidirectsoundbuffer.getcurrentposition(@rep,nil);
          End;
{$ENDIF}
        End;
        GetSb:=Rep;
      End;
    End;
    (*getcurrbuff*)
    5:Begin
      If Joue Then Begin
        Case DevType Of
          0:Begin          // aucune option du mappeur ne permet de faire cette opration -> il faut
            Adr:=GetSb(4); // se taper toute une recherche en se basant sur le flag d'offset des buffers
            Rep:=0;        // on s'arrtera sur le buffer qui passe de "avant"  "aprs" le current mcipos
            Repeat Inc(Rep) Until (Rep>MaxBuffs) Or (PInfoBuff(WaveHeader[Rep-1].dwUser)^.Bytes<=Adr);
            If Rep<=MaxBuffs Then Begin
              For I:=1 To MaxBuffs Do
                If (PInfoBuff(WaveHeader[I-1].dwUser)^.Bytes<=Adr) And (PInfoBuff(WaveHeader[I-1].dwUser)^.Bytes>PInfoBuff(WaveHeader[Rep-1].dwUser)^.Bytes) Then Rep:=I;
              LastBuffer:=Rep;
              GetSb:=Rep;
            End Else Begin
              GetSb:=LastBuffer;
            End;
          End;
          1:Begin
            GetSb:=1;
          End;
{$IFDEF DSound}
          2:Begin
            Adr:=GetSb(4);
            GetSb:=(Adr Div TailleBuffer)+1;
          End;
{$ENDIF}
        End;
      End;
    End;
    (*getcurrofs*)
    6:Begin
      If Joue Then GetSb:=GetSb(4) MOD TailleBuffer;
    End;
    (*getreserve*)
    9:Begin
      If Joue Then Begin
        Case DevType Of
          0:Begin
            Rep:=0;
            For I:=1 To MaxBuffs Do
              If (WaveHeader[I-1].dwFlags And WHDR_DONE)=0 Then Inc(Rep);
            GetSb:=Rep;
          End;
          1:Begin
            GetSb:=0;
          End;
{$IFDEF DSound}
          2:Begin
            curplay:=PInfoBuff(WaveHeader[getsb(5)-1].dwUser)^.Bytes;
            rep:=0;
            for i:=0 to maxbuffs-1 do if PInfoBuff(waveheader[i].dwUser)^.Bytes>curplay then inc(rep);
            getsb:=rep;
          End;
{$ENDIF}
        End;
      End;
    End;
    (*duree*)
    13:Begin
      If Joue Then Begin
        Rep:=GetSb(3);
        If Stereo Then Rep:=Rep Div 2;
        If B16 Then Rep:=Rep Div 2;
        Rep:=Round(Rep/Freq*1000);
        GetSb:=Rep;
      End;
    End;
  End;
End;

Procedure TSound.SndStop;
Var I:Integer;
Var Tmp:LongWord;
    W:Word;
Begin
  If Joue Then Begin
    Case DevType Of
      0:Begin
        waveOutReset(DevHandle); // rinitialise le mappeur
        For I:=1 To MaxBuffs Do Begin
          FreeMem(PInfoBuff(WaveHeader[I-1].dwUser)^.Nfo,InfoSize); // libre la mmoire utilisateur
          Dispose(PInfoBuff(WaveHeader[I-1].dwUser)); // libre la zone info du buffer
          waveOutUnPrepareHeader(DevHandle,@WaveHeader[I-1],SizeOf(WaveHdr)); // sort le buffer du mappeur
          FreeMem(WaveHeader[I-1].lpData); // libre le buffer lui-mme
        End;
        waveOutClose(DevHandle); // coupe le mappeur
      End;
      1:Begin
        seek(OutFile,0); // crit l'en-tte du fichier .wav
        blockwrite(OutFile,'RIFF',4);
        tmp:=ByteCount+36;
        blockwrite(OutFile,tmp,4);
        blockwrite(OutFile,'WAVE',4);
        blockwrite(OutFile,'fmt ',4);
        tmp:=16;
        blockwrite(OutFile,tmp,4);
        w:=1;
        blockwrite(OutFile,w,2);
        w:=1;if stereo then w:=2;
        blockwrite(OutFile,w,2);
        tmp:=freq;
        blockwrite(OutFile,tmp,4);
        tmp:=freq;
        if stereo then tmp:=tmp*2;
        if b16 then tmp:=tmp*2;
        blockwrite(OutFile,tmp,4);
        w:=1;
        if stereo then w:=w*2;
        if b16 then w:=w*2;
        blockwrite(OutFile,w,2);
        w:=16;if not b16 then w:=8;
        blockwrite(OutFile,w,2);
        blockwrite(OutFile,'data',4);
        blockwrite(OutFile,ByteCount,4);
        close(OutFile);
        For I:=1 To MaxBuffs Do Begin
          FreeMem(PInfoBuff(WaveHeader[I-1].dwUser)^.Nfo,InfoSize); // libre la mmoire utilisateur
          Dispose(PInfoBuff(WaveHeader[I-1].dwUser)); // libre la zone info du buffer
          FreeMem(WaveHeader[I-1].lpData); // libre le buffer lui-mme
        End;
      End;
{$IFDEF DSound}
      2:Begin
        lpidirectsoundbuffer.stop; // stoppe le buffer directsound
        For I:=1 To MaxBuffs Do Begin
          FreeMem(PInfoBuff(WaveHeader[I-1].dwUser)^.Nfo,InfoSize); // libre la mmoire utilisateur
          Dispose(PInfoBuff(WaveHeader[I-1].dwUser)); // libre la zone info du buffer
          FreeMem(WaveHeader[I-1].lpData); // libre le buffer lui-mme
        End;
        ppds:=nil; // libre l'objet directsound
      End;
{$ENDIF}
    End;
    Joue:=False;
  End;
End;

Procedure TSound.Do_Wav; //perform
Var I:Integer;
Label Sort;
Var Rep:LongWord;
{$IFDEF DSound}
    Label Sort1;
Var Size1,Size2,CurPos:LongWord;
    CurBuff:Word;
    Pnt1,Pnt2:Pointer;
{$ENDIF}
Begin
  If InWav Or Not Joue Then Exit; // vite un mauvais appel
  InWav:=True;
  Case DevType Of
    0:Begin
      (*rechercher un buffer vide*)
      For I:=1 To MaxBuffs Do Begin
        If (WaveHeader[I-1].dwFlags And WHDR_DONE)<>0 Then Begin
          DecHandler(WaveHeader[I-1].lpData,PInfoBuff(WaveHeader[I-1].dwUser)^.Nfo);
          PInfoBuff(WaveHeader[I-1].dwUser)^.Bytes:=CurrBytes;
          Inc(CurrBytes,TailleBuffer);
          Inc(ByteCount,TailleBuffer);
          rep:=waveoutwrite(DevHandle,@WaveHeader[i-1],sizeof(wavehdr));
          If Rep<>0 Then Begin
            Stop;
            Error(1,Rep,GetError(Rep));
          End;
          Goto Sort;
        end;
      end;
    end;
    1:begin
{$i-}
      If Not IsPause Then Begin
        DecHandler(WaveHeader[0].lpData,PInfoBuff(WaveHeader[0].dwUser)^.Nfo);
        blockwrite(OutFile,WaveHeader[0].lpData^,TailleBuffer);
        Rep:=IOResult;
        if Rep<>0 then begin
          Stop;
          Error(2,Rep,'Error writing to disk');
        end;
        inc(CurrBytes,TailleBuffer);
        inc(ByteCount,TailleBuffer);
      end;
{$i+}
    end;
{$IFDEF DSound}
    2:begin
      lpidirectsoundbuffer.getcurrentposition(@curpos,nil);
      curbuff:=curpos div taillebuffer;

      if (nextbuffertowrite<>curbuff) or inpre then begin // scurit : on n'crit pas sur un buffer en train d'tre jou
        DecHandler(WaveHeader[NextBufferToWrite].lpData,PInfoBuff(WaveHeader[NextBufferToWrite].dwUser)^.Nfo); // on dcode en mmoire
        PInfoBuff(WaveHeader[NextBufferToWrite].dwUser)^.Bytes:=ByteCount;
        lpidirectsoundbuffer.lock(nextbuffertowrite*taillebuffer,taillebuffer,pnt1,size1,pnt2,size2,0);
        if pnt1<>nil then Move(WaveHeader[NextBufferToWrite].lpData^,pnt1^,TailleBuffer); // on transfre vers le directsound
        lpidirectsoundbuffer.unlock(pnt1,size1,pnt2,size2);
        Inc(CurrBytes,TailleBuffer);If CurrBytes=MaxBuffs*TailleBuffer Then CurrBytes:=0;
        Inc(ByteCount,TailleBuffer);
        Inc(NextBufferToWrite);If NextBufferToWrite=MaxBuffs Then NextBufferToWrite:=0;
      End;
    end;
{$ENDIF}
  End;
  Sort:
  InWav:=False;
End;

{Fait tout le job d'initialisation}
Function TSound.Init_Snd:Boolean;
Var I,J:Integer;

Var Rep:LongWord;
    waveHead:Array[1..40] Of Byte;
    Dur:Single;
    WaveX:TWAVEFORMATEX;
{$IFDEF DSound}
    tds:TDSBufferDesc;
    pcmwf:TwaveformatEx;
    pnt1,pnt2:pointer;
    size1,size2:longword;

{$ENDIF}

Begin
  Init_Snd:=False;

  // on transfre tous les "next" vers les valeurs courantes pour l'initialisation
  MaxBuffs:=NextMaxBuffs;
  TailleBuffer:=NextTailleBuffer;
  DevType:=NextDevType;
  DevIndex:=NextDevIndex;
  Freq:=NextFreq;
  Stereo:=NextStereo;
  B16:=NextB16;
  OutFName:=NextOutFName;
  InfoSize:=NextInfoSize;
  Pre:=NextPre;

{$IFDEF DSound}
  If DevType>2 Then Begin
    Error(4,0,'Unknown device type');
    Exit;
  End;
{$ELSE}
  If DevType=2 Then Begin
    Error(5,0,'This version of TSound doesn''t support DirectSound');
    Exit;
  End;
  If DevType>2 Then Begin
    Error(4,0,'Unknown device type');
    Exit;
  End;
{$ENDIF}

  SetLength(WaveHeader,MaxBuffs); // cration de l'espace pour les pointeurs des buffers

  If Joue Then Stop; // on vite des btises
  If Freq=0 Then Exit; // on vite vraiment des btises
  Errh:=False;

  case DevType of
    0:begin
      WaveX.wFormatTag:=WAVE_FORMAT_PCM;
      If Stereo Then WaveX.nChannels:=2 Else WaveX.nChannels:=1;
      WaveX.nSamplesPerSec:=Freq;
      If B16 Then WaveX.wBitsPerSample:=16 Else WaveX.wBitsPerSample:=8;
      WaveX.nBlockAlign:=WaveX.nChannels*WaveX.wBitsPerSample Div 8;
      WaveX.nAvgBytesPerSec:=Freq*WaveX.nBlockAlign;
      WaveX.cbSize:=0;
      Rep:=waveOutOpen(@DevHandle,DevIndex,@WaveX,0,0,0); // ouverture du priph
      If Rep<>0 Then Begin
        Error(1,Rep,GetError(Rep));
        Exit;
      End;
    End;
    1:Begin
{$i-}
      AssignFile(OutFile,OutFName);
      ReWrite(OutFile,1);
      BlockWrite(OutFile,waveHead,44);
{$i+}
      Rep:=IOResult;
      If Rep<>0 Then Begin
        Error(3,Rep,'Error creating file');
      End;
    End;
{$IFDEF DSound}
    2:begin
      rep:=directsoundcreate(nil,ppDS,Nil); // cration de l'objet dsound
      if rep<>ds_ok then begin
        error(6,0,'Error creating DirectSound object : '+errorstring(rep));
        exit;
      end;
      rep:=ppds.SetCooperativeLevel(application.handle,DSSCL_PRIORITY);
      if rep<>ds_ok then begin
        error(6,0,'Error setting up DirectSound cooperative level : '+errorstring(rep));
        exit;
      end;
      tds.dwSize:=sizeof(TDSBufferDesc);
      tds.dwReserved:=0;
      tds.dwflags:=DSBCAPS_PRIMARYBUFFER;
      tds.dwBufferBytes:=maxbuffs*taillebuffer*0;
      tds.lpwfxformat:=nil;

      rep:=ppds.CreateSoundBuffer(tds,lpidirectsoundbufferprimary,nil); // on ouvre un primary pour rgler le format
      if rep<>ds_ok then begin
        error(6,0,'Error creating DirectSound primary buffer : '+errorstring(rep));
        exit;
      end;

      pcmwf.wFormatTag:=WAVE_FORMAT_PCM;
      if stereo then pcmwf.nChannels:=2 else pcmwf.nchannels:=1;
      pcmwf.nSamplesPerSec:=freq;
      if b16 then pcmwf.wBitsPerSample:=16 else pcmwf.wbitspersample:=8;
      pcmwf.nBlockAlign:=pcmwf.wBitsPerSample*pcmwf.nChannels div 8;
      pcmwf.nAvgBytesPerSec:=pcmwf.nSamplesPerSec*pcmwf.nBlockAlign;


      rep:=lpidirectsoundbufferprimary.setformat(pcmwf); // on modifie le format
      if rep<>ds_ok then begin
        error(6,0,'Error while setting DirectSound primary buffer format : '+errorstring(rep));
        exit;
      end;


      tds.dwSize:=sizeof(TDSBufferDesc);
      tds.dwReserved:=0;
      tds.dwflags:=DSBCAPS_GLOBALFOCUS OR DSBCAPS_CTRLFREQUENCY OR DSBCAPS_GETCURRENTPOSITION2 OR DSBCAPS_CTRLVOLUME;
      tds.dwBufferBytes:=maxbuffs*taillebuffer;
      tds.lpwfxFormat:=@pcmwf;
      rep:=ppds.CreateSoundBuffer(tds,lpidirectsoundbuffer,nil); // crons le secondary buffer
      if rep<>ds_ok then begin
        error(6,0,'Error creating DirectSound buffer : '+errorstring(rep));
        exit;
      end;


      lpidirectsoundbuffer.Lock(0,maxbuffs*taillebuffer,pnt1,size1,pnt2,size2,DSBLOCK_ENTIREBUFFER);
      if size1<>0 then fillchar(pnt1^,size1,0); // on le met  blanc
      if size2<>0 then fillchar(pnt2^,size2,0);
      lpidirectsoundbuffer.unLock(pnt1,size1,pnt2,size2);
      nextbuffertowrite:=0;
    end;
{$ENDIF}
  End;
  For I:=1 To MaxBuffs Do WaveHeader[I-1].lpData:=Nil; // on libre ventuellement l'espace buffers
  For I:=1 To MaxBuffs Do Begin
    New(PInfoBuff(WaveHeader[I-1].dwUser)); // pointeur infos
    Getmem(PInfoBuff(WaveHeader[I-1].dwUser)^.Nfo,InfoSize); // infos utilisateur
    FillChar(PInfoBuff(WaveHeader[I-1].dwUser)^.Nfo^,InfoSize,0); // on fill  0
    GetMem(WaveHeader[I-1].lpData,TailleBuffer); // la mmoire du buffer en lui-mme
    WaveHeader[I-1].dwBufferLength:=TailleBuffer;
    WaveHeader[I-1].dwFlags:=0;
    WaveHeader[I-1].reserved:=0;
    PInfoBuff(WaveHeader[I-1].dwUser)^.Bytes:=0;
    If DevType=0 Then Begin
      Rep:=waveOutPrepareHeader(DevHandle,@WaveHeader[I-1],SizeOf(WaveHdr)); // on ajoute ce buffer au mappeur
      If Rep<>0 Then Begin
        Error(1,Rep,GetError(Rep));
        Stop;
        Exit;
      End;
    End;
    WaveHeader[I-1].dwFlags:=WaveHeader[I-1].dwFlags OR WHDR_DONE;
  End;

  Joue:=True;
  IsPause:=False;
  InFlush:=False;
  FlushDoneAction:=0;
  CurrBytes:=0;
  ByteCount:=0;

  SetVolume(NextVol);

  {Impose un pr-buffering}
  InPre:=True;
  SetSb(2,1);
  Dur:=TailleBuffer/Freq;
  If B16 Then Dur:=Dur/2;
  If Stereo Then Dur:=Dur/2;
  J:=Round(Pre/Dur);If J>MaxBuffs Then J:=MaxBuffs;
  For I:=1 To J Do Do_Wav;
  SetSb(2,0);
  InPre:=False;
  Init_Snd:=True;

End;

// handlers vides par dfaut
Procedure DefDecHandler(Pntr,Nfo:Pointer);
Begin
End;

Procedure DefErrHandler(Err1,Err2:LongWord;Txt:String);
Begin
End;

Procedure DefMsgHandler(Msg,Param1,Param2:LongWord);
Begin
End;

// initialisation des valeurs par dfaut et cration de la classe
Constructor TSound.Create;
Begin
  InHerited Create;
  NextVol:=65535;
  DecHandler:=DefDecHandler;
  ErrHandler:=DefErrHandler;
  MsgHandler:=DefMsgHandler;
  Errh:=False;
  InWav:=False;
  InfoSize:=0;
  Stereo:=False;
  B16:=False;
  Freq:=0;
  CurrBytes:=0;
  Joue:=False;
  TailleBuffer:=4096;
  MaxBuffs:=32;
  DevIndex:=LongWord(-1);
  DevHandle:=0;
  DevType:=0;
  ByteCount:=0;
  OutFName:='';
  WaveHeader:=Nil;
  IsPause:=False;
  Pre:=0;
  InFlush:=False;
  NextTailleBuffer:=4096;
  NextMaxBuffs:=32;
  NextDevType:=0;
  NextDevIndex:=LongWord(-1);
  NextFreq:=0;
  NextStereo:=False;
  NextB16:=False;
  NextOutFName:='';
  NextInfoSize:=0;
  NextPre:=0;
  LastBuffer:=1;
End;

End.

